@@ -12,6 +12,7 @@ from unittest.mock import patch
 import pytest
 
 from pypdf import PdfReader, mult
+from pypdf._font import FontDescriptor
 from pypdf._text_extraction import set_custom_rtl
 from pypdf._text_extraction._layout_mode._fixed_width_page import text_show_operations
 from pypdf.errors import ParseError, PdfReadError
@@ -120,7 +121,7 @@ def test_issue_2336():
 
 
 def test_layout_mode_font_class_to_dict():
-    from pypdf._text_extraction._layout_mode._font import Font
+    from pypdf._text_extraction._layout_mode._font import Font  # noqa: PLC0415
 
     font = Font("foo", space_width=8, encoding="utf-8", char_map={}, font_dictionary={})
     assert Font.to_dict(font) == {
@@ -129,7 +130,24 @@ def test_layout_mode_font_class_to_dict():
         "font_dictionary": {},
         "space_width": 8,
         "subtype": "foo",
-        "width_map": {},
+        "font_descriptor": FontDescriptor(
+            name="Unknown",
+            family="Unknown",
+            weight="Unknown",
+            ascent=700.0,
+            descent=-200.0,
+            cap_height=600.0,
+            x_height=500.0,
+            italic_angle=0.0,
+            flags=32,
+            bbox=(
+                -100.0,
+                -200.0,
+                1000.0,
+                900.0,
+            ),
+            character_widths={},
+        ),
         "interpretable": True,
     }
 
@@ -414,3 +432,58 @@ def test_rotated_layout_mode(caplog):
     assert not caplog.records, "No warnings should be issued"
     assert text, "Text matching the page rotation should be extracted"
     assert re.search(r"\r?\n +69\r?\n +UNCLASSIFIED$", text), "Contents should be in expected layout"
+
+
+@pytest.mark.enable_socket
+@pytest.mark.filterwarnings("ignore::pypdf.errors.PdfReadWarning")
+def test_extract_text__none_objects():
+    url = "https://github.com/user-attachments/files/18381726/tika-957721.pdf"
+    name = "tika-957721.pdf"
+    reader = PdfReader(BytesIO(get_data_from_url(url, name=name)))
+
+    reader.pages[0].extract_text()
+    reader.pages[8].extract_text()
+
+
+@pytest.mark.enable_socket
+def test_extract_text__with_visitor_text():
+    def visitor_text(*args, **kwargs):  # noqa: ANN002, ANN003, ANN202
+        pass
+
+    url = "https://github.com/user-attachments/files/18381718/tika-952016.pdf"
+    name = "tika-952016.pdf"
+    stream = BytesIO(get_data_from_url(url, name=name))
+    reader = PdfReader(stream)
+    page = reader.pages[0]
+    page.extract_text(visitor_text=visitor_text)
+
+    reader = PdfReader(BytesIO(get_data_from_url(name="TextAttack_paper.pdf")))
+    page = reader.pages[0]
+    page.extract_text(visitor_text=visitor_text)
+
+
+@pytest.mark.enable_socket
+def test_extract_text__restore_cm_stack_pop_error():
+    url = "https://github.com/user-attachments/files/18381737/tika-966635.pdf"
+    name = "tika-966635.pdf"
+    stream = BytesIO(get_data_from_url(url, name=name))
+    reader = PdfReader(stream)
+    page = reader.pages[10]
+
+    # There is a previous error we already omit ("pop from empty list"), thus
+    # check for the message explicitly here.
+    with pytest.raises(IndexError, match="list index out of range"):
+        page.extract_text()
+
+
+@pytest.mark.timeout(60)
+@pytest.mark.enable_socket
+def test_slow_huge_string():
+    """Tests for #3541"""
+    url = "https://github.com/user-attachments/files/23855795/file.pdf"
+    name = "issue-3541.pdf"
+    stream = BytesIO(get_data_from_url(url, name=name))
+    reader = PdfReader(stream)
+    page = reader.pages[0]
+
+    _ = page.extract_text(extraction_mode="layout")
